@@ -7,9 +7,9 @@ Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 import re
 import logging
 import sys
+import threading
 from collections import MutableMapping
 from contextlib import closing
-from Pyro4.threadutil import Lock
 from Pyro4.errors import NamingError
 
 try:
@@ -29,8 +29,6 @@ except ImportError:
     sqlite3 = None
 
 
-__all__ = ["SqlStorage", "DbmStorage"]
-
 log = logging.getLogger("Pyro4.naming_storage")
 
 
@@ -154,8 +152,9 @@ class SqlStorage(MutableMapping):
                 db.execute("PRAGMA foreign_keys=ON")
                 db.execute("DELETE FROM pyro_metadata")
                 db.execute("DELETE FROM pyro_names")
-                db.execute("VACUUM")
                 db.commit()
+            with closing(sqlite3.connect(self.dbfile, isolation_level=None)) as db:
+                db.execute("VACUUM")  # this cannot run inside a transaction.
         except sqlite3.DatabaseError as e:
             raise NamingError("sqlite error in clear: "+str(e))
 
@@ -251,7 +250,7 @@ class DbmStorage(MutableMapping):
         self.dbmfile = dbmfile
         db = dbm.open(self.dbmfile, "c", mode=0o600)
         db.close()
-        self.lock = Lock()
+        self.lock = threading.Lock()
 
     def __getattr__(self, item):
         raise NotImplementedError("DbmStorage doesn't implement method/attribute '"+item+"'")
@@ -348,8 +347,7 @@ class DbmStorage(MutableMapping):
     def optimized_regex_list(self, regex, return_metadata=False):
         try:
             regex = re.compile(regex + "$")  # add end of string marker
-        except re.error:
-            x = sys.exc_info()[1]
+        except re.error as x:
             raise NamingError("invalid regex: " + str(x))
         with self.lock:
             try:
