@@ -9,13 +9,19 @@ import errno
 import time
 import sys
 import select
-import Pyro4.constants
+from Pyro4.configuration import config
 from Pyro4.errors import CommunicationError, TimeoutError, ConnectionClosedError
 
+try:
+    InterruptedError()  # new since Python 3.4
+except NameError:
+    class InterruptedError(Exception):
+        pass
+
 
 # Note: other interesting errnos are EPERM, ENOBUFS, EMFILE
 # but it seems to me that all these signify an unrecoverable situation.
-# So I didn't include them in de list of retryable errors.
+# So I didn't include them in the list of retryable errors.
 ERRNO_RETRIES = [errno.EINTR, errno.EAGAIN, errno.EWOULDBLOCK, errno.EINPROGRESS]
 if hasattr(errno, "WSAEINTR"):
     ERRNO_RETRIES.append(errno.WSAEINTR)
@@ -67,7 +73,7 @@ def getIpAddress(hostname, workaround127=False, ipVersion=None):
     Returns the IP address for the given host. If you enable the workaround,
     it will use a little hack if the ip address is found to be the loopback address.
     The hack tries to discover an externally visible ip address instead (this only works for ipv4 addresses).
-    Set ipVersion=6 to return ipv6 addresses, 4 to return ipv4, 0 to let OS choose the best one or None to use Pyro4.config.PREFER_IP_VERSION.
+    Set ipVersion=6 to return ipv6 addresses, 4 to return ipv4, 0 to let OS choose the best one or None to use config.PREFER_IP_VERSION.
     """
 
     def getaddr(ipVersion):
@@ -87,9 +93,9 @@ def getIpAddress(hostname, workaround127=False, ipVersion=None):
     try:
         if hostname and ':' in hostname and ipVersion is None:
             ipVersion = 0
-        return getaddr(Pyro4.config.PREFER_IP_VERSION) if ipVersion is None else getaddr(ipVersion)
+        return getaddr(config.PREFER_IP_VERSION) if ipVersion is None else getaddr(ipVersion)
     except socket.gaierror:
-        if ipVersion == 6 or (ipVersion is None and Pyro4.config.PREFER_IP_VERSION == 6):
+        if ipVersion == 6 or (ipVersion is None and config.PREFER_IP_VERSION == 6):
             raise socket.error("unable to determine IPV6 address")
         return getaddr(0)
 
@@ -126,10 +132,7 @@ def receiveData(sock, size):
         retrydelay = 0.0
         msglen = 0
         chunks = []
-        EMPTY_BYTES = b""
-        if sys.platform == "cli":
-            EMPTY_BYTES = ""
-        if Pyro4.config.USE_MSG_WAITALL:
+        if config.USE_MSG_WAITALL:
             # waitall is very convenient and if a socket error occurs,
             # we can assume the receive has failed. No need for a loop,
             # unless it is a retryable error.
@@ -147,8 +150,7 @@ def receiveData(sock, size):
                     break
                 except socket.timeout:
                     raise TimeoutError("receiving: timeout")
-                except socket.error:
-                    x = sys.exc_info()[1]
+                except socket.error as x:
                     err = getattr(x, "errno", x.args[0])
                     if err not in ERRNO_RETRIES:
                         raise ConnectionClosedError("receiving: connection lost: " + str(x))
@@ -164,7 +166,7 @@ def receiveData(sock, size):
                         break
                     chunks.append(chunk)
                     msglen += len(chunk)
-                data = EMPTY_BYTES.join(chunks)
+                data = b"".join(chunks)
                 del chunks
                 if len(data) != size:
                     err = ConnectionClosedError("receiving: not enough data")
@@ -198,8 +200,7 @@ def sendData(sock, data):
             return
         except socket.timeout:
             raise TimeoutError("sending: timeout")
-        except socket.error:
-            x = sys.exc_info()[1]
+        except socket.error as x:
             raise ConnectionClosedError("sending: connection lost: " + str(x))
     else:
         # Socket is in non-blocking mode, use regular send loop.
@@ -210,8 +211,7 @@ def sendData(sock, data):
                 data = data[sent:]
             except socket.timeout:
                 raise TimeoutError("sending: timeout")
-            except socket.error:
-                x = sys.exc_info()[1]
+            except socket.error as x:
                 err = getattr(x, "errno", x.args[0])
                 if err not in ERRNO_RETRIES:
                     raise ConnectionClosedError("sending: connection lost: " + str(x))
@@ -232,7 +232,7 @@ def createSocket(bind=None, connect=None, reuseaddr=False, keepalive=True, timeo
     """
     if bind and connect:
         raise ValueError("bind and connect cannot both be specified at the same time")
-    forceIPv6 = ipv6 or (ipv6 is None and Pyro4.config.PREFER_IP_VERSION == 6)
+    forceIPv6 = ipv6 or (ipv6 is None and config.PREFER_IP_VERSION == 6)
     if isinstance(bind, basestring) or isinstance(connect, basestring):
         family = socket.AF_UNIX
     elif not bind and not connect:
@@ -298,16 +298,20 @@ def createSocket(bind=None, connect=None, reuseaddr=False, keepalive=True, timeo
             xv = sys.exc_info()[1]
             errno = getattr(xv, "errno", 0)
             if errno in ERRNO_RETRIES:
-                if timeout is _GLOBAL_DEFAULT_TIMEOUT:
-                    timeout = None
-                timeout = max(0.1, timeout)  # avoid polling behavior with timeout=0
+                if timeout is _GLOBAL_DEFAULT_TIMEOUT or timeout < 0.1:
+                    timeout = 0.1
                 while True:
-                    sr, sw, se = select.select([], [sock], [sock], timeout)
+                    try:
+                        sr, sw, se = select.select([], [sock], [sock], timeout)
+                    except InterruptedError:
+                        continue
                     if sock in sw:
                         break  # yay, writable now, connect() completed
                     elif sock in se:
+                        sock.close()  # close the socket that refused to connect
                         raise socket.error("connect failed")
             else:
+                sock.close()  # close the socket that refused to connect
                 raise
     if keepalive:
         setKeepalive(sock)
@@ -320,7 +324,7 @@ def createBroadcastSocket(bind=None, reuseaddr=False, timeout=_GLOBAL_DEFAULT_TI
     Set ipv6=True to create an IPv6 socket rather than IPv4.
     Set ipv6=None to use the PREFER_IP_VERSION config setting.
     """
-    forceIPv6 = ipv6 or (ipv6 is None and Pyro4.config.PREFER_IP_VERSION == 6)
+    forceIPv6 = ipv6 or (ipv6 is None and config.PREFER_IP_VERSION == 6)
     if not bind:
         family = socket.AF_INET6 if forceIPv6 else socket.AF_INET
     elif type(bind) is tuple:
@@ -431,6 +435,12 @@ class SocketConnection(object):
     def __del__(self):
         self.close()
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
+
     def send(self, data):
         sendData(self.sock, data)
 
@@ -440,12 +450,13 @@ class SocketConnection(object):
     def close(self):
         try:
             self.sock.shutdown(socket.SHUT_RDWR)
-        except (OSError, socket.error):
+        except:
             pass
         try:
             self.sock.close()
-        except AttributeError:
+        except:
             pass
+        self.pyroInstances = {}   # force releasing the session instances
 
     def fileno(self):
         return self.sock.fileno()
@@ -463,12 +474,13 @@ def findProbablyUnusedPort(family=socket.AF_INET, socktype=socket.SOCK_STREAM):
     """Returns an unused port that should be suitable for binding (likely, but not guaranteed).
     This code is copied from the stdlib's test.test_support module."""
     tempsock = socket.socket(family, socktype)
-    port = bindOnUnusedPort(tempsock)
-    tempsock.close()
-    del tempsock
-    if sys.platform == "cli":
-        return port + 1  # the actual port is somehow still in use by the socket when using IronPython
-    return port
+    try:
+        port = bindOnUnusedPort(tempsock)
+        if sys.platform == "cli":
+            return port + 1  # the actual port is somehow still in use by the socket when using IronPython
+        return port
+    finally:
+        tempsock.close()
 
 
 def bindOnUnusedPort(sock, host='localhost'):
@@ -476,7 +488,10 @@ def bindOnUnusedPort(sock, host='localhost'):
     This code is based on the code in the stdlib's test.test_support module."""
     if sock.family in (socket.AF_INET, socket.AF_INET6) and sock.type == socket.SOCK_STREAM:
         if hasattr(socket, "SO_EXCLUSIVEADDRUSE"):
-            sock.setsockopt(socket.SOL_SOCKET, socket.SO_EXCLUSIVEADDRUSE, 1)
+            try:
+                sock.setsockopt(socket.SOL_SOCKET, socket.SO_EXCLUSIVEADDRUSE, 1)
+            except socket.error:
+                pass
     if sock.family == socket.AF_INET:
         if host == 'localhost':
             sock.bind(('127.0.0.1', 0))
@@ -492,19 +507,18 @@ def bindOnUnusedPort(sock, host='localhost'):
     return sock.getsockname()[1]
 
 
-"""is select() available?"""
-hasSelect = select and hasattr(select, "select")
-
-"""is poll() available?"""
-hasPoll = select and hasattr(select, "poll")
-
-
-def triggerSocket(sock):
-    """send a small data packet over the socket, to trigger it"""
+def interruptSocket(address):
+    """bit of a hack to trigger a blocking server to get out of the loop, useful at clean shutdowns"""
     try:
-        data = b"!" * 16
-        if sys.platform == "cli":
-            data = "!" * 16
-        sock.sendall(data)
-    except (socket.error, AttributeError):
+        sock = createSocket(connect=address, keepalive=False, timeout=None)
+        try:
+            sock.sendall(b"!" * 16)
+        except (socket.error, AttributeError):
+            pass
+        try:
+            sock.shutdown(socket.SHUT_RDWR)
+        except (OSError, socket.error):
+            pass
+        sock.close()
+    except socket.error:
         pass
